@@ -19,6 +19,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include "flash.h"
 #include "programmer.h"
 
@@ -183,6 +184,7 @@ static int print_supported_chips(void)
 		dev = malloc(strlen(chip->name) + 1);
 		if (dev == NULL) {
 			msg_gerr("Out of memory!\n");
+			free(ven);
 			return 1;
 		}
 		strcpy(dev, chip->name);
@@ -314,20 +316,20 @@ static int print_supported_chips(void)
 #if CONFIG_INTERNAL == 1
 static void print_supported_chipsets(void)
 {
-	int i, chipsetcount = 0;
+	unsigned int i, chipsetcount = 0;
 	const struct penable *c = chipset_enables;
-	int maxvendorlen = strlen("Vendor") + 1;
-	int maxchipsetlen = strlen("Chipset") + 1;
+	size_t maxvendorlen = strlen("Vendor") + 1;
+	size_t maxchipsetlen = strlen("Chipset") + 1;
 
 	for (c = chipset_enables; c->vendor_name != NULL; c++) {
 		chipsetcount++;
-		maxvendorlen = max(maxvendorlen, strlen(c->vendor_name));
-		maxchipsetlen = max(maxchipsetlen, strlen(c->device_name));
+		maxvendorlen = MAX(maxvendorlen, strlen(c->vendor_name));
+		maxchipsetlen = MAX(maxchipsetlen, strlen(c->device_name));
 	}
 	maxvendorlen++;
 	maxchipsetlen++;
 
-	msg_ginfo("Supported chipsets (total: %d):\n\n", chipsetcount);
+	msg_ginfo("Supported chipsets (total: %u):\n\n", chipsetcount);
 
 	msg_ginfo("Vendor");
 	for (i = strlen("Vendor"); i < maxvendorlen; i++)
@@ -354,12 +356,12 @@ static void print_supported_chipsets(void)
 static void print_supported_boards_helper(const struct board_info *boards,
 				   const char *devicetype)
 {
-	int i;
+	unsigned int i;
 	unsigned int boardcount_good = 0, boardcount_bad = 0, boardcount_nt = 0;
 	const struct board_match *e = board_matches;
 	const struct board_info *b = boards;
-	int maxvendorlen = strlen("Vendor") + 1;
-	int maxboardlen = strlen("Board") + 1;
+	size_t maxvendorlen = strlen("Vendor") + 1;
+	size_t maxboardlen = strlen("Board") + 1;
 
 	for (b = boards; b->vendor != NULL; b++) {
 		maxvendorlen = max(maxvendorlen, strlen(b->vendor));
@@ -423,7 +425,7 @@ static void print_supported_boards_helper(const struct board_info *boards,
 }
 #endif
 
-void print_supported_devs(const struct programmer_entry prog, const char *const type)
+static void print_supported_devs(const struct programmer_entry prog, const char *const type)
 {
 	const struct dev_entry *const devs = prog.devs.dev;
 	msg_ginfo("\nSupported %s devices for the %s programmer:\n", type, prog.name);
@@ -583,77 +585,77 @@ const struct board_info boards_known[] = {
 	B("ASRock",	"P4i65G",		OK, "http://www.asrock.com/mb/overview.asp?Model=P4i65G", NULL),
 	B("ASRock",	"P4i65GV",		OK, "http://www.asrock.com/mb/overview.asp?Model=P4i65GV", NULL),
 	B("ASRock",	"Z68 Extreme4",		BAD, "http://www.asrock.com/mb/overview.asp?Model=Z68%20Extreme4", "Probing works (Winbond W25Q64, 8192 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
-	B("ASUS",	"A7N8X Deluxe",		OK, "http://www.asus.com/Motherboards/AMD_Socket_A/A7N8X_Deluxe/", NULL),
-	B("ASUS",	"A7N8X-E Deluxe",	OK, "http://www.asus.com/Motherboards/AMD_Socket_A/A7N8XE_Deluxe/", NULL),
-	B("ASUS",	"A7N8X-VM/400",		OK, "http://www.asus.com/Motherboards/AMD_Socket_A/A7N8XVM400/", NULL),
+	B("ASUS",	"A7N8X Deluxe",		OK, "https://www.asus.com/Motherboards/AMD_Socket_A/A7N8X_Deluxe/", NULL),
+	B("ASUS",	"A7N8X-E Deluxe",	OK, "https://www.asus.com/Motherboards/AMD_Socket_A/A7N8XE_Deluxe/", NULL),
+	B("ASUS",	"A7N8X-VM/400",		OK, "https://www.asus.com/Motherboards/AMD_Socket_A/A7N8XVM400/", NULL),
 	B("ASUS",	"A7V133",		OK, NULL, NULL),
 	B("ASUS",	"A7V333",		OK, NULL, NULL),
-	B("ASUS",	"A7V400-MX",		OK, "http://www.asus.com/Motherboards/AMD_Socket_A/A7V400MX/", NULL),
-	B("ASUS",	"A7V600-X",		OK, "http://www.asus.com/Motherboards/AMD_Socket_A/A7V600X/", NULL),
-	B("ASUS",	"A7V8X",		OK, "http://www.asus.com/Motherboards/AMD_Socket_A/A7V8X/", NULL),
-	B("ASUS",	"A7V8X-MX",		OK, "http://www.asus.com/Motherboards/AMD_Socket_A/A7V8XMX/", NULL),
-	B("ASUS",	"A7V8X-MX SE",		OK, "http://www.asus.com/Motherboards/AMD_Socket_A/A7V8XMX_SE/", NULL),
-	B("ASUS",	"A7V8X-X",		OK, "http://www.asus.com/Motherboards/AMD_Socket_A/A7V8XX/", NULL),
+	B("ASUS",	"A7V400-MX",		OK, "https://www.asus.com/Motherboards/AMD_Socket_A/A7V400MX/", NULL),
+	B("ASUS",	"A7V600-X",		OK, "https://www.asus.com/Motherboards/AMD_Socket_A/A7V600X/", NULL),
+	B("ASUS",	"A7V8X",		OK, "https://www.asus.com/Motherboards/AMD_Socket_A/A7V8X/", NULL),
+	B("ASUS",	"A7V8X-MX",		OK, "https://www.asus.com/Motherboards/AMD_Socket_A/A7V8XMX/", NULL),
+	B("ASUS",	"A7V8X-MX SE",		OK, "https://www.asus.com/Motherboards/AMD_Socket_A/A7V8XMX_SE/", NULL),
+	B("ASUS",	"A7V8X-X",		OK, "https://www.asus.com/Motherboards/AMD_Socket_A/A7V8XX/", NULL),
 	B("ASUS",	"A8M2N-LA (NodusM3-GL8E)", OK, "http://h10010.www1.hp.com/ewfrf/wc/document?docname=c00757531&cc=us&dlc=en&lc=en", "This is an OEM board from HP, the HP name is NodusM3-GL8E."),
-	B("ASUS",	"A8N-E",		OK, "http://www.asus.com/Motherboards/AMD_Socket_939/A8NE/", NULL),
+	B("ASUS",	"A8N-E",		OK, "https://www.asus.com/Motherboards/AMD_Socket_939/A8NE/", NULL),
 	B("ASUS",	"A8N-LA (Nagami-GL8E)",	OK, "http://h10025.www1.hp.com/ewfrf/wc/document?lc=en&cc=us&docname=c00647121&dlc=en", "This is an OEM board from HP, the HP name is Nagami-GL8E."),
-	B("ASUS",	"A8N-SLI",		OK, "http://www.asus.com/Motherboards/AMD_Socket_939/A8NSLI/", NULL),
+	B("ASUS",	"A8N-SLI",		OK, "https://www.asus.com/Motherboards/AMD_Socket_939/A8NSLI/", NULL),
 	B("ASUS",	"A8N-SLI Deluxe",	NT, NULL, "Should work out of the box since r1593."),
-	B("ASUS",	"A8N-SLI Premium",	OK, "http://www.asus.com/Motherboards/AMD_Socket_939/A8NSLI_Premium/", NULL),
-	B("ASUS",	"A8N-VM",		OK, "http://www.asus.com/Motherboards/AMD_Socket_939/A8NVM/", NULL),
-	B("ASUS",	"A8N-VM CSM",		OK, "http://www.asus.com/Motherboards/AMD_Socket_939/A8NVM_CSM/", NULL),
+	B("ASUS",	"A8N-SLI Premium",	OK, "https://www.asus.com/Motherboards/AMD_Socket_939/A8NSLI_Premium/", NULL),
+	B("ASUS",	"A8N-VM",		OK, "https://www.asus.com/Motherboards/AMD_Socket_939/A8NVM/", NULL),
+	B("ASUS",	"A8N-VM CSM",		OK, "https://www.asus.com/Motherboards/AMD_Socket_939/A8NVM_CSM/", NULL),
 	B("ASUS",	"A8NE-FM/S",		OK, "http://www.hardwareschotte.de/hardware/preise/proid_1266090/preis_ASUS+A8NE-FM", NULL),
-	B("ASUS",	"A8V Deluxe",		OK, "http://www.asus.com/Motherboards/AMD_Socket_939/A8V_Deluxe/", NULL),
-	B("ASUS",	"A8V-E Deluxe",		OK, "http://www.asus.com/Motherboards/AMD_Socket_939/A8VE_Deluxe/", NULL),
-	B("ASUS",	"A8V-E SE",		OK, "http://www.asus.com/Motherboards/AMD_Socket_939/A8VE_SE/", "See http://www.coreboot.org/pipermail/coreboot/2007-October/026496.html"),
-	B("ASUS",	"C60M1-I",		OK, "http://www.asus.com/Motherboards/C60M1I/", "The MAC address of the onboard network card is stored in flash."),
-	B("ASUS",	"Crosshair II Formula",	OK, "http://www.asus.com/Motherboards/AMD_AM2Plus/Crosshair_II_Formula/", NULL),
-	B("ASUS",	"Crosshair IV Extreme",	OK, "http://www.asus.com/Motherboards/AMD_AM3/Crosshair_IV_Extreme/", NULL),
+	B("ASUS",	"A8V Deluxe",		OK, "https://www.asus.com/Motherboards/AMD_Socket_939/A8V_Deluxe/", NULL),
+	B("ASUS",	"A8V-E Deluxe",		OK, "https://www.asus.com/Motherboards/AMD_Socket_939/A8VE_Deluxe/", NULL),
+	B("ASUS",	"A8V-E SE",		OK, "https://www.asus.com/Motherboards/AMD_Socket_939/A8VE_SE/", "See http://www.coreboot.org/pipermail/coreboot/2007-October/026496.html"),
+	B("ASUS",	"C60M1-I",		OK, "https://www.asus.com/Motherboards/C60M1I/", "The MAC address of the onboard network card is stored in flash."),
+	B("ASUS",	"Crosshair II Formula",	OK, "https://www.asus.com/Motherboards/AMD_AM2Plus/Crosshair_II_Formula/", NULL),
+	B("ASUS",	"Crosshair IV Extreme",	OK, "https://www.asus.com/Motherboards/AMD_AM3/Crosshair_IV_Extreme/", NULL),
 	B("ASUS",	"CUSL2-C",		OK, NULL, "The image provided by ASUS is only 256 kB big and has to be written to the upper 256 kB of the 512 kB chip."),
-	B("ASUS",	"DSAN-DX",		NT, "http://www.asus.com/Server_Workstation/Server_Motherboards/DSANDX/", NULL),
-	B("ASUS",	"E35M1-I DELUXE",	OK, "http://www.asus.com/Motherboards/AMD_CPU_on_Board/E35M1I_DELUXE/", NULL),
-	B("ASUS",	"F1A75-V PRO",		OK, "http://www.asus.com/Motherboard/F1A75V_PRO/", NULL),
+	B("ASUS",	"DSAN-DX",		NT, "https://www.asus.com/Server_Workstation/Server_Motherboards/DSANDX/", NULL),
+	B("ASUS",	"E35M1-I DELUXE",	OK, "https://www.asus.com/Motherboards/AMD_CPU_on_Board/E35M1I_DELUXE/", NULL),
+	B("ASUS",	"F1A75-V PRO",		OK, "https://www.asus.com/Motherboard/F1A75V_PRO/", NULL),
 	B("ASUS",	"F2A85-M",		DEP, "https://www.asus.com/Motherboards/F2A85M/", "UEFI builds v6404 and above disable access to some parts of the flash, cf. http://www.coreboot.org/ASUS_F2A85-M#UEFI_builds_that_allow_flash_chip_access"),
-	B("ASUS",	"K8N",			OK, "http://www.asus.com/Motherboards/AMD_Socket_754/K8N/", NULL),
-	B("ASUS",	"K8V",			OK, "http://www.asus.com/Motherboards/AMD_Socket_754/K8V/", NULL),
-	B("ASUS",	"K8V SE Deluxe",	OK, "http://www.asus.com/Motherboards/AMD_Socket_754/K8V_SE_Deluxe/", NULL),
-	B("ASUS",	"K8V-X",		OK, "http://www.asus.com/Motherboards/AMD_Socket_754/K8VX/", NULL),
-	B("ASUS",	"K8V-X SE",		OK, "http://www.asus.com/Motherboards/AMD_Socket_754/K8VX_SE/", NULL),
-	B("ASUS",	"KFSN4-DRE/SAS",	OK, "http://www.asus.com/Server_Workstation/Server_Motherboards/KFSN4DRESAS/", NULL),
-	B("ASUS",	"M2A-MX",		OK, "http://www.asus.com/Motherboards/AMD_AM2/M2AMX/", NULL),
-	B("ASUS",	"M2A-VM (HDMI)",	OK, "http://www.asus.com/Motherboards/AMD_AM2/M2AVM/", NULL),
-	B("ASUS",	"M2N32-SLI Deluxe",	OK, "http://www.asus.com/Motherboards/AMD_AM2/M2N32SLI_DeluxeWireless_Edition/", NULL),
-	B("ASUS",	"M2N68-VM",		OK, "http://www.asus.com/Motherboards/AMD_AM2Plus/M2N68VM/", NULL),
-	B("ASUS",	"M2NBP-VM CSM",		OK, "http://www.asus.com/Motherboards/AMD_AM2/M2NBPVM_CSM/", NULL),
-	B("ASUS",	"M2N-E",		OK, "http://www.asus.com/Motherboards/AMD_AM2/M2NE/", "If the machine doesn't come up again after flashing, try resetting the NVRAM(CMOS). The MAC address of the onboard network card will change to the value stored in the new image, so backup the old address first. See https://flashrom.org/pipermail/flashrom/2009-November/000879.html"),
-	B("ASUS",	"M2N-E SLI",		OK, "http://www.asus.com/Motherboards/AMD_AM2/M2NE_SLI/", NULL),
-	B("ASUS",	"M2N-MX SE Plus",	OK, "http://www.asus.com/Motherboards/M2NMX_SE_Plus/", NULL),
-	B("ASUS",	"M2NPV-VM",		OK, "http://www.asus.com/Motherboards/AMD_AM2/M2NPVVM/", NULL),
-	B("ASUS",	"M2N-SLI Deluxe",	OK, "http://www.asus.com/Motherboards/AMD_AM2/M2NSLI_Deluxe/", NULL),
-	B("ASUS",	"M2V",			OK, "http://www.asus.com/Motherboards/AMD_AM2/M2V/", NULL),
-	B("ASUS",	"M2V-MX",		OK, "http://www.asus.com/Motherboards/AMD_AM2/M2VMX/", NULL),
-	B("ASUS",	"M3A",			OK, "http://www.asus.com/Motherboards/AMD_AM2Plus/M3A/", NULL),
-	B("ASUS",	"M3A76-CM",		OK, "http://www.asus.com/Motherboards/AMD_AM2Plus/M3A76CM/", NULL),
-	B("ASUS",	"M3A78-EH",		OK, "http://www.asus.com/Motherboards/AMD_AM2Plus/M3A78EH/", NULL),
-	B("ASUS",	"M3A78-EM",		OK, "http://www.asus.com/Motherboards/AMD_AM2Plus/M3A78EM/", NULL),
-	B("ASUS",	"M3N78 PRO",		OK, "http://www.asus.com/Motherboards/AMD_AM2Plus/M3N78_PRO/", NULL),
-	B("ASUS",	"M3N78-VM",		OK, "http://www.asus.com/Motherboards/AMD_AM2Plus/M3N78VM/", NULL),
-	B("ASUS",	"M3N-H/HDMI",		OK, "http://www.asus.com/Motherboards/M3NHHDMI//", NULL),
-	B("ASUS",	"M4A785TD-M EVO",	OK, "http://www.asus.com/Motherboards/AMD_AM3/M4A785TDM_EVO/", NULL),
-	B("ASUS",	"M4A785TD-V EVO",	OK, "http://www.asus.com/Motherboards/AMD_AM3/M4A785TDV_EVO/", NULL),
-	B("ASUS",	"M4A785T-M",		OK, "http://www.asus.com/Motherboards/AMD_AM3/M4A785TM/", NULL),
-	B("ASUS",	"M4A78-EM",		OK, "http://www.asus.com/Motherboards/AMD_AM2Plus/M4A78EM/", NULL),
-	B("ASUS",	"M4A78LT-M LE",		OK, "http://www.asus.com/Motherboards/AMD_AM3/M4A78LTM_LE/", NULL),
-	B("ASUS",	"M4A79T Deluxe",	OK, "http://www.asus.com/Motherboards/AMD_AM3/M4A79T_Deluxe/", NULL),
-	B("ASUS",	"M4A87TD/USB3",		OK, "http://www.asus.com/Motherboards/AMD_AM3/M4A87TDUSB3/", NULL),
-	B("ASUS",	"M4A89GTD PRO",		OK, "http://www.asus.com/Motherboards/AMD_AM3/M4A89GTD_PRO/", NULL),
-	B("ASUS",	"M4N68T V2",		OK, "http://www.asus.com/Motherboards/AMD_AM3/M4N68T_V2/", NULL),
-	B("ASUS",	"M4N78 PRO",		OK, "http://www.asus.com/Motherboards/AMD_AM2Plus/M4N78_PRO/", NULL),
-	B("ASUS",	"M4N78 SE",		OK, "http://www.asus.com/Motherboards/AMD_AM2Plus/M4N78_SE/", NULL),
-	B("ASUS",	"M5A78L-M LX",		OK, "http://www.asus.com/Motherboards/AMD_AM3Plus/M5A78LM_LX/", "The MAC address of the onboard LAN NIC is stored in flash, hence overwritten by flashrom; see https://flashrom.org/pipermail/flashrom/2012-May/009200.html"),
-	B("ASUS",	"M5A97 (rev. 1.0)",	OK, "http://www.asus.com/Motherboard/M5A97/", NULL),
-	B("ASUS",	"M5A99X EVO",		OK, "http://www.asus.com/Motherboards/AMD_AM3Plus/M5A99X_EVO/", NULL),
-	B("ASUS",	"Maximus IV Extreme",	BAD, "http://www.asus.com/Motherboards/Intel_Socket_1155/Maximus_IV_Extreme/", "Probing works (Macronix MX25L3205, 4096 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
+	B("ASUS",	"K8N",			OK, "https://www.asus.com/Motherboards/AMD_Socket_754/K8N/", NULL),
+	B("ASUS",	"K8V",			OK, "https://www.asus.com/Motherboards/AMD_Socket_754/K8V/", NULL),
+	B("ASUS",	"K8V SE Deluxe",	OK, "https://www.asus.com/Motherboards/AMD_Socket_754/K8V_SE_Deluxe/", NULL),
+	B("ASUS",	"K8V-X",		OK, "https://www.asus.com/Motherboards/AMD_Socket_754/K8VX/", NULL),
+	B("ASUS",	"K8V-X SE",		OK, "https://www.asus.com/Motherboards/AMD_Socket_754/K8VX_SE/", NULL),
+	B("ASUS",	"KFSN4-DRE/SAS",	OK, "https://www.asus.com/Server_Workstation/Server_Motherboards/KFSN4DRESAS/", NULL),
+	B("ASUS",	"M2A-MX",		OK, "https://www.asus.com/Motherboards/AMD_AM2/M2AMX/", NULL),
+	B("ASUS",	"M2A-VM (HDMI)",	OK, "https://www.asus.com/Motherboards/AMD_AM2/M2AVM/", NULL),
+	B("ASUS",	"M2N32-SLI Deluxe",	OK, "https://www.asus.com/Motherboards/AMD_AM2/M2N32SLI_DeluxeWireless_Edition/", NULL),
+	B("ASUS",	"M2N68-VM",		OK, "https://www.asus.com/Motherboards/AMD_AM2Plus/M2N68VM/", NULL),
+	B("ASUS",	"M2NBP-VM CSM",		OK, "https://www.asus.com/Motherboards/AMD_AM2/M2NBPVM_CSM/", NULL),
+	B("ASUS",	"M2N-E",		OK, "https://www.asus.com/Motherboards/AMD_AM2/M2NE/", "If the machine doesn't come up again after flashing, try resetting the NVRAM(CMOS). The MAC address of the onboard network card will change to the value stored in the new image, so backup the old address first. See https://flashrom.org/pipermail/flashrom/2009-November/000879.html"),
+	B("ASUS",	"M2N-E SLI",		OK, "https://www.asus.com/Motherboards/AMD_AM2/M2NE_SLI/", NULL),
+	B("ASUS",	"M2N-MX SE Plus",	OK, "https://www.asus.com/Motherboards/M2NMX_SE_Plus/", NULL),
+	B("ASUS",	"M2NPV-VM",		OK, "https://www.asus.com/Motherboards/AMD_AM2/M2NPVVM/", NULL),
+	B("ASUS",	"M2N-SLI Deluxe",	OK, "https://www.asus.com/Motherboards/AMD_AM2/M2NSLI_Deluxe/", NULL),
+	B("ASUS",	"M2V",			OK, "https://www.asus.com/Motherboards/AMD_AM2/M2V/", NULL),
+	B("ASUS",	"M2V-MX",		OK, "https://www.asus.com/Motherboards/AMD_AM2/M2VMX/", NULL),
+	B("ASUS",	"M3A",			OK, "https://www.asus.com/Motherboards/AMD_AM2Plus/M3A/", NULL),
+	B("ASUS",	"M3A76-CM",		OK, "https://www.asus.com/Motherboards/AMD_AM2Plus/M3A76CM/", NULL),
+	B("ASUS",	"M3A78-EH",		OK, "https://www.asus.com/Motherboards/AMD_AM2Plus/M3A78EH/", NULL),
+	B("ASUS",	"M3A78-EM",		OK, "https://www.asus.com/Motherboards/AMD_AM2Plus/M3A78EM/", NULL),
+	B("ASUS",	"M3N78 PRO",		OK, "https://www.asus.com/Motherboards/AMD_AM2Plus/M3N78_PRO/", NULL),
+	B("ASUS",	"M3N78-VM",		OK, "https://www.asus.com/Motherboards/AMD_AM2Plus/M3N78VM/", NULL),
+	B("ASUS",	"M3N-H/HDMI",		OK, "https://www.asus.com/Motherboards/M3NHHDMI//", NULL),
+	B("ASUS",	"M4A785TD-M EVO",	OK, "https://www.asus.com/Motherboards/AMD_AM3/M4A785TDM_EVO/", NULL),
+	B("ASUS",	"M4A785TD-V EVO",	OK, "https://www.asus.com/Motherboards/AMD_AM3/M4A785TDV_EVO/", NULL),
+	B("ASUS",	"M4A785T-M",		OK, "https://www.asus.com/Motherboards/AMD_AM3/M4A785TM/", NULL),
+	B("ASUS",	"M4A78-EM",		OK, "https://www.asus.com/Motherboards/AMD_AM2Plus/M4A78EM/", NULL),
+	B("ASUS",	"M4A78LT-M LE",		OK, "https://www.asus.com/Motherboards/AMD_AM3/M4A78LTM_LE/", NULL),
+	B("ASUS",	"M4A79T Deluxe",	OK, "https://www.asus.com/Motherboards/AMD_AM3/M4A79T_Deluxe/", NULL),
+	B("ASUS",	"M4A87TD/USB3",		OK, "https://www.asus.com/Motherboards/AMD_AM3/M4A87TDUSB3/", NULL),
+	B("ASUS",	"M4A89GTD PRO",		OK, "https://www.asus.com/Motherboards/AMD_AM3/M4A89GTD_PRO/", NULL),
+	B("ASUS",	"M4N68T V2",		OK, "https://www.asus.com/Motherboards/AMD_AM3/M4N68T_V2/", NULL),
+	B("ASUS",	"M4N78 PRO",		OK, "https://www.asus.com/Motherboards/AMD_AM2Plus/M4N78_PRO/", NULL),
+	B("ASUS",	"M4N78 SE",		OK, "https://www.asus.com/Motherboards/AMD_AM2Plus/M4N78_SE/", NULL),
+	B("ASUS",	"M5A78L-M LX",		OK, "https://www.asus.com/Motherboards/AMD_AM3Plus/M5A78LM_LX/", "The MAC address of the onboard LAN NIC is stored in flash, hence overwritten by flashrom; see https://flashrom.org/pipermail/flashrom/2012-May/009200.html"),
+	B("ASUS",	"M5A97 (rev. 1.0)",	OK, "https://www.asus.com/Motherboard/M5A97/", NULL),
+	B("ASUS",	"M5A99X EVO",		OK, "https://www.asus.com/Motherboards/AMD_AM3Plus/M5A99X_EVO/", NULL),
+	B("ASUS",	"Maximus IV Extreme",	BAD, "https://www.asus.com/Motherboards/Intel_Socket_1155/Maximus_IV_Extreme/", "Probing works (Macronix MX25L3205, 4096 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
 	B("ASUS",	"MEW-AM",		BAD, NULL, "No public report found. Owned by Uwe Hermann <uwe@hermann-uwe.de>. May work now."),
 	B("ASUS",	"MEW-VM",		BAD, "http://www.elhvb.com/mboards/OEM/HP/manual/ASUS%20MEW-VM.htm", "No public report found. Owned by Uwe Hermann <uwe@hermann-uwe.de>. May work now."),
 	B("ASUS",	"OPLX-M",		NT, NULL, "Untested board enable."),
@@ -669,61 +671,62 @@ const struct board_info boards_known[] = {
 	B("ASUS",	"P4B266",		OK, NULL, NULL),
 	B("ASUS",	"P4B266-LM",		OK, "http://esupport.sony.com/US/perl/swu-list.pl?mdl=PCVRX650", NULL),
 	B("ASUS",	"P4B533-E",		OK, NULL, NULL),
-	B("ASUS",	"P4C800-E Deluxe",	OK, "http://www.asus.com/Motherboards/Intel_Socket_478/P4C800E_Deluxe/", NULL),
+	B("ASUS",	"P4C800-E Deluxe",	OK, "https://www.asus.com/Motherboards/Intel_Socket_478/P4C800E_Deluxe/", NULL),
 	B("ASUS",	"P4GV-LA (Guppy)",	OK, "http://h20000.www2.hp.com/bizsupport/TechSupport/Document.jsp?objectID=c00363478", NULL),
-	B("ASUS",	"P4P800",		OK, "http://www.asus.com/Motherboards/Intel_Socket_478/P4P800/", NULL),
-	B("ASUS",	"P4P800-E Deluxe",	OK, "http://www.asus.com/Motherboards/Intel_Socket_478/P4P800E_Deluxe/", NULL),
-	B("ASUS",	"P4P800-VM",		OK, "http://www.asus.com/Motherboards/Intel_Socket_478/P4P800VM/", NULL),
-	B("ASUS",	"P4P800-X",		OK, "http://www.asus.com/Motherboards/Intel_Socket_478/P4P800X/", NULL),
-	B("ASUS",	"P4PE-X/TE",		NT, "http://www.asus.com/999/html/events/mb/socket478/p4pe-x-te/overview.htm", NULL),
+	B("ASUS",	"P4P800",		OK, "https://www.asus.com/Motherboards/Intel_Socket_478/P4P800/", NULL),
+	B("ASUS",	"P4P800-E Deluxe",	OK, "https://www.asus.com/Motherboards/Intel_Socket_478/P4P800E_Deluxe/", NULL),
+	B("ASUS",	"P4P800-VM",		OK, "https://www.asus.com/Motherboards/Intel_Socket_478/P4P800VM/", NULL),
+	B("ASUS",	"P4P800-X",		OK, "https://www.asus.com/Motherboards/Intel_Socket_478/P4P800X/", NULL),
+	B("ASUS",	"P4P800SE",		OK, "https://www.asus.com/supportonly/P4P800 SE/", NULL),
+	B("ASUS",	"P4PE-X/TE",		NT, "https://www.asus.com/999/html/events/mb/socket478/p4pe-x-te/overview.htm", NULL),
 	B("ASUS",	"P4S533-X",		OK, NULL, NULL),
-	B("ASUS",	"P4S800-MX",		OK, "http://www.asus.com/Motherboards/Intel_Socket_478/P4S800MX/", NULL),
+	B("ASUS",	"P4S800-MX",		OK, "https://www.asus.com/Motherboards/Intel_Socket_478/P4S800MX/", NULL),
 	B("ASUS",	"P4SC-E",		OK, NULL, "Part of ASUS Terminator P4 533 barebone system"),
 	B("ASUS",	"P4SD-LA",		OK, "http://h10025.www1.hp.com/ewfrf/wc/document?docname=c00022505", NULL),
 	B("ASUS",	"P5A",			OK, NULL, NULL),
 	B("ASUS",	"P5B",			OK, NULL, NULL),
-	B("ASUS",	"P5B-Deluxe",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5B_Deluxe/", NULL),
+	B("ASUS",	"P5B-Deluxe",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5B_Deluxe/", NULL),
 	B("ASUS",	"P5B-VM",		OK, NULL, NULL),
 	B("ASUS",	"P5BV-M",		BAD, NULL, "Reported by Bernhard M. Wiedemann <bernhard@uml12d.zq1.de> to flashrom@coreboot.org, no public archive. Missing board enable and/or SST49LF008A unlocking. May work now."),
-	B("ASUS",	"P5BV-R",		OK, "http://www.asus.com/Server_Workstation/Servers/RS120E5PA2/", "Used in RS120-E5/PA2 servers."),
-	B("ASUS",	"P5GC-MX/1333",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5GCMX1333/", NULL),
-	B("ASUS",	"P5GD1 Pro",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5GD1_PRO/", NULL),
+	B("ASUS",	"P5BV-R",		OK, "https://www.asus.com/Server_Workstation/Servers/RS120E5PA2/", "Used in RS120-E5/PA2 servers."),
+	B("ASUS",	"P5GC-MX/1333",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5GCMX1333/", NULL),
+	B("ASUS",	"P5GD1 Pro",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5GD1_PRO/", NULL),
 	B("ASUS",	"P5GD1-VM/S",		OK, NULL, "This is an OEM board from FSC. Although flashrom supports it and can probably not distinguish it from the P5GD1-VM, please note that the P5GD1-VM BIOS does not support the FSC variants completely."),
 	B("ASUS",	"P5GD1(-VM)",		NT, NULL, "Untested board enable."),
-	B("ASUS",	"P5GD2 Premium",	OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5GD2_Premium/", NULL),
-	B("ASUS",	"P5GD2-X",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5GD2X/", NULL),
-	B("ASUS",	"P5GDC Deluxe",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5GDC_Deluxe/", NULL),
-	B("ASUS",	"P5GDC-V Deluxe",	OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5GDCV_Deluxe/", NULL),
+	B("ASUS",	"P5GD2 Premium",	OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5GD2_Premium/", NULL),
+	B("ASUS",	"P5GD2-X",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5GD2X/", NULL),
+	B("ASUS",	"P5GDC Deluxe",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5GDC_Deluxe/", NULL),
+	B("ASUS",	"P5GDC-V Deluxe",	OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5GDCV_Deluxe/", NULL),
 	B("ASUS",	"P5GD2/C variants",	NT, NULL, "Untested board enable."),
-	B("ASUS",	"P5K SE",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5K_SE/", NULL),
-	B("ASUS",	"P5K-V",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5KV/", NULL),
-	B("ASUS",	"P5K-VM",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5KVM/", NULL),
-	B("ASUS",	"P5KC",			OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5KC/", NULL),
+	B("ASUS",	"P5K SE",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5K_SE/", NULL),
+	B("ASUS",	"P5K-V",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5KV/", NULL),
+	B("ASUS",	"P5K-VM",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5KVM/", NULL),
+	B("ASUS",	"P5KC",			OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5KC/", NULL),
 	B("ASUS",	"P5KPL-AM IN/GB",	OK, "http://support.asus.com/download.aspx?SLanguage=en&m=P5KPL-AM+IN%2fGB&os=29", NULL),
-	B("ASUS",	"P5KPL-CM",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5KPLCM/", NULL),
-	B("ASUS",	"P5KPL-VM",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5KPLVM/", "Found in V3-P5G31."),
-	B("ASUS",	"P5L-MX",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5LMX/", NULL),
-	B("ASUS",	"P5L-VM 1394",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5LVM_1394/", NULL),
+	B("ASUS",	"P5KPL-CM",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5KPLCM/", NULL),
+	B("ASUS",	"P5KPL-VM",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5KPLVM/", "Found in V3-P5G31."),
+	B("ASUS",	"P5L-MX",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5LMX/", NULL),
+	B("ASUS",	"P5L-VM 1394",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5LVM_1394/", NULL),
 	B("ASUS",	"P5LD2",		OK, NULL, NULL),
 	B("ASUS",	"P5LD2-MQ",		OK, "http://support.asus.com/download.aspx?SLanguage=en&p=8&s=12&m=Vintage-PH2&os=&hashedid=n/a", "Found in ASUS Vintage-PH2 barebones."),
-	B("ASUS",	"P5LD2-VM",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5LD2VM/", NULL),
-	B("ASUS",	"P5LD2-VM DH",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5LD2VM_DH/", NULL),
+	B("ASUS",	"P5LD2-VM",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5LD2VM/", NULL),
+	B("ASUS",	"P5LD2-VM DH",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5LD2VM_DH/", NULL),
 	B("ASUS",	"P5LP-LE (Lithium-UL8E)", OK, "http://h10025.www1.hp.com/ewfrf/wc/document?docname=c00379616&tmp_task=prodinfoCategory&cc=us&dlc=en&lc=en&product=1159887", "This is an OEM board from HP."),
 	B("ASUS",	"P5LP-LE (Epson OEM)",	OK, NULL, "This is an OEM board from Epson (e.g. Endeavor MT7700)."),
 	B("ASUS",	"P5LP-LE",		NT, NULL, "This designation is used for OEM boards from HP, Epson and maybe others. The HP names vary and not all of them have been tested yet. Please report any success or failure, thanks."),
-	B("ASUS",	"P5N-D",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5ND/", NULL),
-	B("ASUS",	"P5N-E SLI",		NT, "http://www.asus.com/Motherboards/Intel_Socket_775/P5NE_SLI/", "Untested board enable."),
-	B("ASUS",	"P5N32-E SLI",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5N32E_SLI/", NULL),
-	B("ASUS",	"P5N7A-VM",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5N7AVM/", NULL),
-	B("ASUS",	"P5ND2-SLI Deluxe",	OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5ND2SLI_Deluxe/", NULL),
-	B("ASUS",	"P5PE-VM",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5PEVM/", NULL),
-	B("ASUS",	"P5QPL-AM",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5QPLAM/", NULL),
-	B("ASUS",	"P5VD1-X",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5VD1X/", NULL),
-	B("ASUS",	"P5VD2-MX",		OK, "http://www.asus.com/Motherboards/Intel_Socket_775/P5VD2MX/", "The MAC address of the onboard LAN NIC is stored in flash, hence overwritten by flashrom; see https://flashrom.org/pipermail/flashrom/2012-March/009014.html"),
-	B("ASUS",	"P6T SE",		OK, "http://www.asus.com/Motherboards/Intel_Socket_1366/P6T_SE/", NULL),
-	B("ASUS",	"P6T Deluxe",		OK, "http://www.asus.com/Motherboards/Intel_Socket_1366/P6T_Deluxe/", NULL),
-	B("ASUS",	"P6T Deluxe V2",	OK, "http://www.asus.com/Motherboards/Intel_Socket_1366/P6T_Deluxe_V2/", NULL),
-	B("ASUS",	"P7H57D-V EVO",		OK, "http://www.asus.com/Motherboards/Intel_Socket_1156/P7H57DV_EVO/", NULL),
+	B("ASUS",	"P5N-D",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5ND/", NULL),
+	B("ASUS",	"P5N-E SLI",		NT, "https://www.asus.com/Motherboards/Intel_Socket_775/P5NE_SLI/", "Untested board enable."),
+	B("ASUS",	"P5N32-E SLI",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5N32E_SLI/", NULL),
+	B("ASUS",	"P5N7A-VM",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5N7AVM/", NULL),
+	B("ASUS",	"P5ND2-SLI Deluxe",	OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5ND2SLI_Deluxe/", NULL),
+	B("ASUS",	"P5PE-VM",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5PEVM/", NULL),
+	B("ASUS",	"P5QPL-AM",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5QPLAM/", NULL),
+	B("ASUS",	"P5VD1-X",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5VD1X/", NULL),
+	B("ASUS",	"P5VD2-MX",		OK, "https://www.asus.com/Motherboards/Intel_Socket_775/P5VD2MX/", "The MAC address of the onboard LAN NIC is stored in flash, hence overwritten by flashrom; see https://flashrom.org/pipermail/flashrom/2012-March/009014.html"),
+	B("ASUS",	"P6T SE",		OK, "https://www.asus.com/Motherboards/Intel_Socket_1366/P6T_SE/", NULL),
+	B("ASUS",	"P6T Deluxe",		OK, "https://www.asus.com/Motherboards/Intel_Socket_1366/P6T_Deluxe/", NULL),
+	B("ASUS",	"P6T Deluxe V2",	OK, "https://www.asus.com/Motherboards/Intel_Socket_1366/P6T_Deluxe_V2/", NULL),
+	B("ASUS",	"P7H57D-V EVO",		OK, "https://www.asus.com/Motherboards/Intel_Socket_1156/P7H57DV_EVO/", NULL),
 	B("ASUS",	"P7H55-M LX",		BAD, NULL, "flashrom works correctly, but GbE LAN is nonworking (probably due to a missing/bogus MAC address; see https://flashrom.org/pipermail/flashrom/2011-July/007432.html and http://ubuntuforums.org/showthread.php?t=1534389 for a possible workaround)"),
 	B("ASUS",	"P8B-E/4L",		BAD, NULL, "Probing works (Winbond W25Q64, 8192 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
 	B("ASUS",	"P8B WS",		BAD, NULL, "Probing works (Winbond W25Q32, 4096 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
@@ -731,24 +734,24 @@ const struct board_info boards_known[] = {
 	B("ASUS",	"P8H61 PRO",		BAD, NULL, "Probing works (Winbond W25Q32, 4096 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
 	B("ASUS",	"P8H61-M LE/USB3",	BAD, NULL, "Probing works (Winbond W25Q32, 4096 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
 	B("ASUS",	"P8H67-M PRO",		BAD, NULL, "Probing works (Macronix MX25L3205, 4096 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."), // some firmware versions apparently are not locked, see report by Marek Zakrzewski
-	B("ASUS",	"P8H77-I",		OK, "http://www.asus.com/Motherboards/P8H77I/", NULL),
-	B("ASUS",	"P8H77-M",		OK, "http://www.asus.com/Motherboards/P8H77M/", NULL),
-	B("ASUS",	"P8H77-V LE",		OK, "http://www.asus.com/Motherboards/P8H77V_LE/", NULL),
+	B("ASUS",	"P8H77-I",		OK, "https://www.asus.com/Motherboards/P8H77I/", NULL),
+	B("ASUS",	"P8H77-M",		OK, "https://www.asus.com/Motherboards/P8H77M/", NULL),
+	B("ASUS",	"P8H77-V LE",		OK, "https://www.asus.com/Motherboards/P8H77V_LE/", NULL),
 	B("ASUS",	"P8P67 (rev. 3.1)",	BAD, NULL, "Probing works (Macronix MX25L3205, 4096 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
 	B("ASUS",	"P8P67 LE (B2)",	OK, NULL, NULL),
 	B("ASUS",	"P8P67 LE (B3)",	BAD, NULL, "Probing works (Winbond W25Q32, 4096 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
-	B("ASUS",	"P8P67 PRO (rev. 3.0)",	OK, "http://www.asus.com/Motherboards/Intel_Socket_1155/P8P67_PRO/", NULL),
+	B("ASUS",	"P8P67 PRO (rev. 3.0)",	OK, "https://www.asus.com/Motherboards/Intel_Socket_1155/P8P67_PRO/", NULL),
 	B("ASUS",	"P8P67-M PRO",		BAD, NULL, "Probing works (Macronix MX25L3205, 4096 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
-	B("ASUS",	"P8Z68-V",		OK, "http://www.asus.com/Motherboards/Intel_Socket_1155/P8Z68V/", "Warning: MAC address of LOM is stored at 0x1000 - 0x1005 of the image."),
+	B("ASUS",	"P8Z68-V",		OK, "https://www.asus.com/Motherboards/Intel_Socket_1155/P8Z68V/", "Warning: MAC address of LOM is stored at 0x1000 - 0x1005 of the image."),
 	B("ASUS",	"P8Z68-V LE",		BAD, NULL, "Probing works (Winbond W25Q64, 8192 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
 	B("ASUS",	"P8Z68-V PRO",		BAD, NULL, "Probing works (Winbond W25Q64, 8192 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
-	B("ASUS",	"P8Z68-V PRO/GEN3",	OK, "http://www.asus.com/Motherboards/Intel_Socket_1155/P8Z68V_PROGEN3/", "Warning: MAC address of LOM is stored at 0x1000 - 0x1005 of the image."),
-	B("ASUS",	"RAMPAGE III GENE",	OK, "http://www.asus.com/Motherboards/RAMPAGE_III_GENE/", "The MAC address of the onboard network card is stored in flash."),
-	B("ASUS",	"SABERTOOTH 990FX",	OK, "http://www.asus.com/Motherboards/AMD_AM3Plus/SABERTOOTH_990FX/", NULL),
-	B("ASUS",	"SABERTOOTH 990FX R2.0", OK, "http://www.asus.com/Motherboards/AMD_AM3Plus/SABERTOOTH_990FX_R20/", NULL),
+	B("ASUS",	"P8Z68-V PRO/GEN3",	OK, "https://www.asus.com/Motherboards/Intel_Socket_1155/P8Z68V_PROGEN3/", "Warning: MAC address of LOM is stored at 0x1000 - 0x1005 of the image."),
+	B("ASUS",	"RAMPAGE III GENE",	OK, "https://www.asus.com/Motherboards/RAMPAGE_III_GENE/", "The MAC address of the onboard network card is stored in flash."),
+	B("ASUS",	"SABERTOOTH 990FX",	OK, "https://www.asus.com/Motherboards/AMD_AM3Plus/SABERTOOTH_990FX/", NULL),
+	B("ASUS",	"SABERTOOTH 990FX R2.0", OK, "https://www.asus.com/Motherboards/AMD_AM3Plus/SABERTOOTH_990FX_R20/", NULL),
 	B("ASUS",	"TUSL2-C",		NT, "http://support.asus.com/download.aspx?SLanguage=en&p=1&s=4&m=TUSL2-C&os=&hashedid=n/a", "Untested board enable."),
-	B("ASUS",	"Z8NA-D6C",		OK, "http://www.asus.com/Server_Workstation/Server_Motherboards/Z8NAD6C/", NULL),
-	B("ASUS",	"Z8PE-D12",		OK, "http://www.asus.com/Server_Workstation/Server_Motherboards/Z8PED12/", NULL),
+	B("ASUS",	"Z8NA-D6C",		OK, "https://www.asus.com/Server_Workstation/Server_Motherboards/Z8NAD6C/", NULL),
+	B("ASUS",	"Z8PE-D12",		OK, "https://www.asus.com/Server_Workstation/Server_Motherboards/Z8PED12/", NULL),
 	B("Attro",	"G5G100-P",		OK, "http://www.attro.com/motherboard/G5G100-P.htm", NULL),
 	B("Bachmann",	"OT200",		OK, "http://www.bachmann.info/produkte/bedien-und-beobachtungsgeraete/operator-terminals/", NULL),
 	B("BCOM",	"WinNET100",		OK, "http://www.coreboot.org/BCOM_WINNET100", "Used in the IGEL-316 thin client."),
@@ -1146,7 +1149,7 @@ const struct board_info laptops_known[] = {
 	B("Acer",	"Aspire 1520",		OK, "http://support.acer.com/us/en/acerpanam/notebook/0000/Acer/Aspire1520/Aspire1520nv.shtml", NULL),
 	B("Acer",	"Aspire One",		BAD, NULL, "http://www.coreboot.org/pipermail/coreboot/2009-May/048041.html"),
 	B("ASUS",	"A8Jm",			OK, NULL, NULL),
-	B("ASUS",	"Eee PC 701 4G",	BAD, "http://www.asus.com/Eee/Eee_PC/Eee_PC_4G/", "It seems the chip (25X40) is behind some SPI flash translation layer (likely in the EC, the ENE KB3310)."),
+	B("ASUS",	"Eee PC 701 4G",	BAD, "https://www.asus.com/Eee/Eee_PC/Eee_PC_4G/", "It seems the chip (25X40) is behind some SPI flash translation layer (likely in the EC, the ENE KB3310)."),
 	B("ASUS",	"M6Ne",			NT, NULL, "Untested board enable."),
 	B("ASUS",	"U38N",			OK, NULL, NULL),
 	B("Clevo",	"P150HM",		BAD, "http://www.clevo.com.tw/en/products/prodinfo_2.asp?productid=307", "Probing works (Macronix MX25L3205, 4096 kB, SPI), but parts of the flash are problematic: descriptor is r/o (conforming to ICH reqs), ME region is locked."),
